clear all;
close all;
clc;

% Let's retrieve the chickenpox data
data = chickenpox_dataset;
% Convert the cell vector to a row vector
% (Note: columns are samples, rows are features!)
data_vec = ...;

% Plot them for explotatory analysis
figure(1);
plot(...);
xlabel('Month');
ylabel('Number of cases');
title('Monthly cases of chickenpox');

% Percentage of data used for LTSM training
train_fraction = ...;
% Number of training samples
num_steps_train = ...;
% Split the data into training ...
data_train = data_vec(...);
% ... and testing dataset
data_test = data_vec(...);

% We will not use normalized data for the time being, but in future we will,
% probably (what about zscore()?)
% [data_train, mu, sigma] = ...;

% Let's train a model what performs one step prediction
x_train = data_train(...);
y_train = data_train(...);

% Definition of a LSTM model in Matlab
num_features = 1;  % one predictor only
num_responses = 1;  % and one response only
num_hidden_units = ...; % keep the history in 60 (120, 200) internal states
% Definition of LTSM layers
layers = [ ...
    sequenceInputLayer(num_features), ...
    lstmLayer(num_hidden_units), ...
    fullyConnectedLayer(num_responses), ...
    regressionLayer
    ];
% Definition of training parameters
num_epochs = 1000;
options = trainingOptions('adam', ...  % better than gradient descent!
    'MaxEpochs', num_epochs, ...
    'GradientThreshold',1, ...
    'InitialLearnRate',0.005, ...
    'LearnRateSchedule','piecewise', ...
    'LearnRateDropPeriod', floor(2*num_epochs/3), ...
    'LearnRateDropFactor',0.2, ...
    'Verbose',0, ...
    'Plots','training-progress');
% Let's train the model only of not already available
net_file = 'lstm_chickenpox_trained.mat';
if isfile(...)
    ...;
else
    net = trainNetwork(...);
    save(net_file, 'net');
end

% We have the model, let's try prediction ...
% But first we need to initialise the LTSM network by running it through the
% known training data. This will set its initial internal state.
net = predictAndUpdateState(...);
% And predict the first unknown output
[net, y_predicted] = predictAndUpdateState(..., 'ExecutionEnvironment', 'cpu');

% For the next steps we will feed the model with modelled predictions, not with measurements
num_test_elems = length(data_test(1:end-1));
for i = 2:num_test_elems
    [net, y_predicted(:,i)] = predictAndUpdateState(..., 'ExecutionEnvironment', 'cpu');
end

% To compare it with test dataset we need to denormalise first
% Normalisation: data_test = (data_test - mu)/sigma;
% y_predicted = ...;

% Plot the prediction and real data
figure(2);
plot(1:num_test_elems, data_test(2:end), 1:num_test_elems, y_predicted);
legend('Observation', 'Prediction');
xlabel('Month');
ylabel('Number of cases');
title('Prediction of monthly cases of chickenpox');
